@@ -43,18 +43,19 @@ import static org.codehaus.groovy.ast.tools.WideningCategories.*;
  * A class to write out the optimized statements
  */
 public class OptimizingStatementWriter extends StatementWriter {
-    
+
     private static class FastPathData {
         private Label pathStart = new Label();
         private Label afterPath = new Label();
     }
-    
+
     public static class ClassNodeSkip{}
-    
+
     public static class StatementMeta {
         private boolean optimize=false;
         protected MethodNode target;
         protected ClassNode type;
+        protected VariableExpression declaredVariableExpression;
         protected boolean[] involvedTypes = new boolean[typeMapKeyNames.length];
         public void chainInvolvedTypes(OptimizeFlagsCollector opt) {
             for (int i=0; i<typeMapKeyNames.length; i++) {
@@ -83,7 +84,7 @@ public class OptimizingStatementWriter extends StatementWriter {
         MethodCaller.newStatic(BytecodeInterface8.class, "isOrigF"),
         MethodCaller.newStatic(BytecodeInterface8.class, "isOrigZ"),
     };
-    
+
     private static final MethodCaller disabledStandardMetaClass = MethodCaller.newStatic(BytecodeInterface8.class, "disabledStandardMetaClass");
     private boolean fastPathBlocked = false;
     private WriterController controller;
@@ -92,26 +93,26 @@ public class OptimizingStatementWriter extends StatementWriter {
         super(controller);
         this.controller = controller;
     }
-    
+
     private boolean notEnableFastPath(StatementMeta meta) {
         // return false if cannot do fast path and if are already on the path
         return fastPathBlocked || meta==null || !meta.optimize || controller.isFastPath();
     }
-    
+
     private FastPathData writeGuards(StatementMeta meta, Statement statement) {
         if (notEnableFastPath(meta)) return null;
         controller.getAcg().onLineNumber(statement, null);
         MethodVisitor mv = controller.getMethodVisitor();
         FastPathData fastPathData = new FastPathData();
         Label slowPath = new Label();
-        
+
         for (int i=0; i<guards.length; i++) {
             if (meta.involvedTypes[i]) {
                 guards[i].call(mv);
                 mv.visitJumpInsn(IFEQ, slowPath);
             }
         }
-        
+
         // meta class check with boolean holder
         String owner = BytecodeHelper.getClassInternalName(controller.getClassNode());
         MethodNode mn = controller.getMethodNode();
@@ -119,37 +120,37 @@ public class OptimizingStatementWriter extends StatementWriter {
             mv.visitFieldInsn(GETSTATIC, owner, Verifier.STATIC_METACLASS_BOOL, "Z");
             mv.visitJumpInsn(IFNE, slowPath);
         }
-        
+
         //standard metaclass check
         disabledStandardMetaClass.call(mv);
         mv.visitJumpInsn(IFNE, slowPath);
-        
+
         // other guards here
-        
+
         mv.visitJumpInsn(GOTO, fastPathData.pathStart);
         mv.visitLabel(slowPath);
-        
+
         return fastPathData;
     }
-        
+
     private void writeFastPathPrelude(FastPathData meta) {
         MethodVisitor mv = controller.getMethodVisitor();
         mv.visitJumpInsn(GOTO, meta.afterPath);
         mv.visitLabel(meta.pathStart);
         controller.switchToFastPath();
     }
-    
+
     private void writeFastPathEpilogue(FastPathData meta) {
         MethodVisitor mv = controller.getMethodVisitor();
         mv.visitLabel(meta.afterPath);
         controller.switchToSlowPath();
     }
-    
+
     @Override
     public void writeBlockStatement(BlockStatement statement) {
         StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
         FastPathData fastPathData = writeGuards(meta, statement);
-        
+
         if (fastPathData==null) {
             // normal mode with different paths
             // important is to not to have a fastpathblock here,
@@ -162,13 +163,13 @@ public class OptimizingStatementWriter extends StatementWriter {
             fastPathBlocked = true;
             super.writeBlockStatement(statement);
             fastPathBlocked = oldFastPathBlock;
-            
+
             writeFastPathPrelude(fastPathData);
             super.writeBlockStatement(statement);
             writeFastPathEpilogue(fastPathData);
         }
     }
-    
+
     @Override
     public void writeDoWhileLoop(DoWhileStatement statement) {
         if (controller.isFastPath()) {
@@ -176,19 +177,19 @@ public class OptimizingStatementWriter extends StatementWriter {
         } else {
             StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
             FastPathData fastPathData = writeGuards(meta, statement);
-    
+
             boolean oldFastPathBlock = fastPathBlocked;
             fastPathBlocked = true;
             super.writeDoWhileLoop(statement);
             fastPathBlocked = oldFastPathBlock;
-            
+
             if (fastPathData==null) return;
             writeFastPathPrelude(fastPathData);
             super.writeDoWhileLoop(statement);
             writeFastPathEpilogue(fastPathData);
         }
     }
-    
+
     @Override
     protected void writeIteratorHasNext(MethodVisitor mv) {
         if (controller.isFastPath()) {
@@ -197,7 +198,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             super.writeIteratorHasNext(mv);
         }
     }
-    
+
     @Override
     protected void writeIteratorNext(MethodVisitor mv) {
         if (controller.isFastPath()) {
@@ -206,7 +207,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             super.writeIteratorNext(mv);
         }
     }
-    
+
     @Override
     protected void writeForInLoop(ForStatement statement) {
         if (controller.isFastPath()) {
@@ -214,12 +215,12 @@ public class OptimizingStatementWriter extends StatementWriter {
         } else {
             StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
             FastPathData fastPathData = writeGuards(meta, statement);
-    
+
             boolean oldFastPathBlock = fastPathBlocked;
             fastPathBlocked = true;
             super.writeForInLoop(statement);
             fastPathBlocked = oldFastPathBlock;
-            
+
             if (fastPathData==null) return;
             writeFastPathPrelude(fastPathData);
             super.writeForInLoop(statement);
@@ -234,12 +235,12 @@ public class OptimizingStatementWriter extends StatementWriter {
         } else {
             StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
             FastPathData fastPathData = writeGuards(meta, statement);
-            
+
             boolean oldFastPathBlock = fastPathBlocked;
             fastPathBlocked = true;
             super.writeForLoopWithClosureList(statement);
             fastPathBlocked = oldFastPathBlock;
-            
+
             if (fastPathData==null) return;
             writeFastPathPrelude(fastPathData);
             super.writeForLoopWithClosureList(statement);
@@ -254,12 +255,12 @@ public class OptimizingStatementWriter extends StatementWriter {
         } else {
             StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
             FastPathData fastPathData = writeGuards(meta, statement);
-            
+
             boolean oldFastPathBlock = fastPathBlocked;
             fastPathBlocked = true;
             super.writeWhileLoop(statement);
             fastPathBlocked = oldFastPathBlock;
-            
+
             if (fastPathData==null) return;
             writeFastPathPrelude(fastPathData);
             super.writeWhileLoop(statement);
@@ -269,24 +270,24 @@ public class OptimizingStatementWriter extends StatementWriter {
 
     @Override
     public void writeIfElse(IfStatement statement) {
-        if (controller.isFastPath()) {
+        StatementMeta meta = statement.getNodeMetaData(StatementMeta.class);
+        FastPathData fastPathData = writeGuards(meta, statement);
+
+        if (fastPathData==null) {
             super.writeIfElse(statement);
         } else {
-            StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
-            FastPathData fastPathData = writeGuards(meta, statement);
-            
             boolean oldFastPathBlock = fastPathBlocked;
             fastPathBlocked = true;
             super.writeIfElse(statement);
             fastPathBlocked = oldFastPathBlock;
-            
-            if (fastPathData==null) return;
+
+            if (fastPathData == null) return;
             writeFastPathPrelude(fastPathData);
             super.writeIfElse(statement);
             writeFastPathEpilogue(fastPathData);
         }
     }
-    
+
     private boolean isNewPathFork(StatementMeta meta) {
         // meta.optimize -> can do fast path
         if (meta==null || meta.optimize==false) return false;
@@ -296,7 +297,7 @@ public class OptimizingStatementWriter extends StatementWriter {
         if (controller.isFastPath()) return false;
         return true;
     }
-    
+
     @Override
     public void writeReturn(ReturnStatement statement) {
         if (controller.isFastPath()) {
@@ -304,17 +305,21 @@ public class OptimizingStatementWriter extends StatementWriter {
         } else {
             StatementMeta meta = (StatementMeta) statement.getNodeMetaData(StatementMeta.class);
             if (isNewPathFork(meta) && writeDeclarationExtraction(statement)) {
+                if (meta.declaredVariableExpression != null) {
+                    // declaration was replaced by assignment so we need to define the variable
+                    controller.getCompileStack().defineVariable(meta.declaredVariableExpression, false);
+                }
                 FastPathData fastPathData = writeGuards(meta, statement);
-                
+
                 boolean oldFastPathBlock = fastPathBlocked;
                 fastPathBlocked = true;
                 super.writeReturn(statement);
                 fastPathBlocked = oldFastPathBlock;
-                
+
                 if (fastPathData==null) return;
                 writeFastPathPrelude(fastPathData);
                 super.writeReturn(statement);
-                writeFastPathEpilogue(fastPathData); 
+                writeFastPathEpilogue(fastPathData);
             } else {
                 super.writeReturn(statement);
             }
@@ -340,19 +345,23 @@ public class OptimizingStatementWriter extends StatementWriter {
             // (3) fast path possible and in slow or fastPath. Nothing to do here.
             //
             // the only case we need to handle is then (2).
-            
+
             if (isNewPathFork(meta) && writeDeclarationExtraction(statement)) {
+                if (meta.declaredVariableExpression != null) {
+                    // declaration was replaced by assignment so we need to define the variable
+                    controller.getCompileStack().defineVariable(meta.declaredVariableExpression, false);
+                }
                 FastPathData fastPathData = writeGuards(meta, statement);
-                
+
                 boolean oldFastPathBlock = fastPathBlocked;
                 fastPathBlocked = true;
                 super.writeExpressionStatement(statement);
                 fastPathBlocked = oldFastPathBlock;
-                
+
                 if (fastPathData==null) return;
                 writeFastPathPrelude(fastPathData);
                 super.writeExpressionStatement(statement);
-                writeFastPathEpilogue(fastPathData);            
+                writeFastPathEpilogue(fastPathData);
             } else {
                 super.writeExpressionStatement(statement);
             }
@@ -366,17 +375,22 @@ public class OptimizingStatementWriter extends StatementWriter {
             ex = rs.getExpression();
         } else if (statement instanceof ExpressionStatement) {
             ExpressionStatement es = (ExpressionStatement) statement;
-            ex = es.getExpression();            
+            ex = es.getExpression();
         } else {
             throw new GroovyBugError("unknown statement type :"+statement.getClass());
-        } 
+        }
         if (!(ex instanceof DeclarationExpression)) return true;
         DeclarationExpression declaration = (DeclarationExpression) ex;
         ex = declaration.getLeftExpression();
         if (ex instanceof TupleExpression) return false;
-                
-        // do declaration
-        controller.getCompileStack().defineVariable(declaration.getVariableExpression(), false);
+
+        // stash declared variable in case we do subsequent visits after we
+        // change to assignment only
+        StatementMeta meta = statement.getNodeMetaData(StatementMeta.class);
+        if (meta != null) {
+            meta.declaredVariableExpression = declaration.getVariableExpression();
+        }
+
         // change statement to do assignment only
         BinaryExpression assignment = new BinaryExpression(
                 declaration.getLeftExpression(),
@@ -390,18 +404,18 @@ public class OptimizingStatementWriter extends StatementWriter {
             rs.setExpression(assignment);
         } else if (statement instanceof ExpressionStatement) {
             ExpressionStatement es = (ExpressionStatement) statement;
-            es.setExpression(assignment);            
+            es.setExpression(assignment);
         } else {
             throw new GroovyBugError("unknown statement type :"+statement.getClass());
         }
         return true;
     }
-    
+
     public static void setNodeMeta(TypeChooser chooser, ClassNode classNode) {
         if (classNode.getNodeMetaData(ClassNodeSkip.class)!=null) return;
         new OptVisitor(chooser).visitClass(classNode);
     }
-    
+
     private static StatementMeta addMeta(ASTNode node) {
         StatementMeta metaOld = (StatementMeta) node.getNodeMetaData(StatementMeta.class);
         StatementMeta meta = metaOld;
@@ -410,13 +424,13 @@ public class OptimizingStatementWriter extends StatementWriter {
         if (metaOld==null) node.setNodeMetaData(StatementMeta.class, meta);
         return meta;
     }
-    
+
     private static StatementMeta addMeta(ASTNode node, OptimizeFlagsCollector opt) {
         StatementMeta meta = addMeta(node);
         meta.chainInvolvedTypes(opt);
         return meta;
     }
-    
+
     private static class OptimizeFlagsCollector {
         private static class OptimizeFlagsEntry {
             private boolean canOptimize = false;
@@ -488,7 +502,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             current.involvedTypes = new boolean[typeMapKeyNames.length];
         }
     }
-    
+
     private static class OptVisitor extends ClassCodeVisitorSupport {
         private final TypeChooser typeChooser;
 
@@ -503,7 +517,7 @@ public class OptimizingStatementWriter extends StatementWriter {
         private boolean optimizeMethodCall = true;
         private VariableScope scope;
         private static final VariableScope nonStaticScope = new VariableScope();
-        
+
         @Override
         public void visitClass(ClassNode node) {
             this.optimizeMethodCall = !node.implementsInterface(GROOVY_INTERCEPTABLE_TYPE);
@@ -513,20 +527,20 @@ public class OptimizingStatementWriter extends StatementWriter {
             this.scope=null;
             this.node=null;
         }
-        
+
         @Override
         public void visitMethod(MethodNode node) {
             scope = node.getVariableScope();
             super.visitMethod(node);
             opt.reset();
         }
-        
+
         @Override
         public void visitConstructor(ConstructorNode node) {
             scope = node.getVariableScope();
             super.visitConstructor(node);
         }
-        
+
         @Override
         public void visitReturnStatement(ReturnStatement statement) {
             opt.push();
@@ -534,7 +548,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             if (opt.shouldOptimize()) addMeta(statement,opt);
             opt.pop(opt.shouldOptimize());
         }
-        
+
         @Override
         public void visitUnaryMinusExpression(UnaryMinusExpression expression) {
             //TODO: implement int operations for this
@@ -542,7 +556,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             StatementMeta meta = addMeta(expression);
             meta.type = OBJECT_TYPE;
         }
-        
+
         @Override
         public void visitUnaryPlusExpression(UnaryPlusExpression expression) {
             //TODO: implement int operations for this
@@ -550,7 +564,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             StatementMeta meta = addMeta(expression);
             meta.type = OBJECT_TYPE;
         }
-        
+
         @Override
         public void visitBitwiseNegationExpression(BitwiseNegationExpression expression) {
             //TODO: implement int operations for this
@@ -558,7 +572,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             StatementMeta meta = addMeta(expression);
             meta.type = OBJECT_TYPE;
         }
-        
+
         private void addTypeInformation(Expression expression, Expression orig) {
             ClassNode type = typeChooser.resolveType(expression, node);
             if (isPrimitiveType(type)) {
@@ -568,24 +582,24 @@ public class OptimizingStatementWriter extends StatementWriter {
                 opt.chainInvolvedType(type);
             }
         }
-        
+
         @Override
         public void visitPrefixExpression(PrefixExpression expression) {
             super.visitPrefixExpression(expression);
             addTypeInformation(expression.getExpression(),expression);
         }
-        
+
         @Override
         public void visitPostfixExpression(PostfixExpression expression) {
             super.visitPostfixExpression(expression);
             addTypeInformation(expression.getExpression(),expression);
-        }        
-        
+        }
+
         @Override
         public void visitDeclarationExpression(DeclarationExpression expression) {
             Expression right = expression.getRightExpression();
             right.visit(this);
-            
+
             ClassNode leftType = typeChooser.resolveType(expression.getLeftExpression(), node);
             Expression rightExpression = expression.getRightExpression();
             ClassNode rightType = optimizeDivWithIntOrLongTarget(rightExpression, leftType);
@@ -605,23 +619,23 @@ public class OptimizingStatementWriter extends StatementWriter {
                 opt.chainInvolvedType(rightType);
             }
         }
-        
+
         @Override
         public void visitBinaryExpression(BinaryExpression expression) {
             if (expression.getNodeMetaData(StatementMeta.class)!=null) return;
             super.visitBinaryExpression(expression);
-            
+
             ClassNode leftType = typeChooser.resolveType(expression.getLeftExpression(), node);
             ClassNode rightType = typeChooser.resolveType(expression.getRightExpression(), node);
             ClassNode resultType = null;
             int operation = expression.getOperation().getType();
-            
+
             if (operation==Types.LEFT_SQUARE_BRACKET && leftType.isArray()) {
                 opt.chainShouldOptimize(true);
                 resultType = leftType.getComponentType();
             } else {
                 switch (operation) {
-                    case Types.COMPARE_EQUAL: 
+                    case Types.COMPARE_EQUAL:
                     case Types.COMPARE_LESS_THAN:
                     case Types.COMPARE_LESS_THAN_EQUAL:
                     case Types.COMPARE_GREATER_THAN:
@@ -683,7 +697,7 @@ public class OptimizingStatementWriter extends StatementWriter {
                         }
                 }
             }
-            
+
             if (resultType!=null) {
                 StatementMeta meta = addMeta(expression);
                 meta.type = resultType;
@@ -706,7 +720,7 @@ public class OptimizingStatementWriter extends StatementWriter {
 
             ClassNode originalResultType = typeChooser.resolveType(binExp, node);
             if (    !originalResultType.equals(BigDecimal_TYPE) ||
-                    !(isLongCategory(assignmentTartgetType) || isFloatingCategory(assignmentTartgetType)) 
+                    !(isLongCategory(assignmentTartgetType) || isFloatingCategory(assignmentTartgetType))
             ) {
                 return null;
             }
@@ -740,7 +754,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             if (opt.shouldOptimize()) addMeta(statement,opt);
             opt.pop(opt.shouldOptimize());
         }
-        
+
         @Override
         public void visitBlockStatement(BlockStatement block) {
             opt.push();
@@ -755,12 +769,12 @@ public class OptimizingStatementWriter extends StatementWriter {
                 opt.chainCanOptimize(true);
                 opt.pop(true);
             } else {
-                opt.chainShouldOptimize(optAll);                
+                opt.chainShouldOptimize(optAll);
                 if (optAll) addMeta(block,opt);
                 opt.pop(optAll);
             }
         }
-        
+
         @Override
         public void visitIfElse(IfStatement statement) {
             opt.push();
@@ -768,7 +782,13 @@ public class OptimizingStatementWriter extends StatementWriter {
             if (opt.shouldOptimize()) addMeta(statement,opt);
             opt.pop(opt.shouldOptimize());
         }
-        
+
+        /*@Override
+        public void visitConstantExpression(ConstantExpression expression) {
+            super.visitConstantExpression(expression);
+            opt.chainShouldOptimize(true);
+        }*/
+
         @Override
         public void visitStaticMethodCallExpression(StaticMethodCallExpression expression) {
             if (expression.getNodeMetaData(StatementMeta.class)!=null) return;
@@ -776,23 +796,23 @@ public class OptimizingStatementWriter extends StatementWriter {
 
             setMethodTarget(expression,expression.getMethod(), expression.getArguments(), true);
         }
-        
+
         @Override
         public void visitMethodCallExpression(MethodCallExpression expression) {
             if (expression.getNodeMetaData(StatementMeta.class)!=null) return;
             super.visitMethodCallExpression(expression);
-            
+
             Expression object = expression.getObjectExpression();
             boolean setTarget = AsmClassGenerator.isThisExpression(object);
             if (!setTarget) {
                 if (!(object instanceof ClassExpression)) return;
                 setTarget = object.equals(node);
             }
-            
+
             if (!setTarget) return;
             setMethodTarget(expression, expression.getMethodAsString(), expression.getArguments(), true);
         }
-        
+
         @Override
         public void visitConstructorCallExpression(ConstructorCallExpression call) {
             if (call.getNodeMetaData(StatementMeta.class)!=null) return;
@@ -802,7 +822,7 @@ public class OptimizingStatementWriter extends StatementWriter {
             // check the meta class of the other class
             // setMethodTarget(call, "<init>", call.getArguments(), false);
         }
-        
+
         private void setMethodTarget(Expression expression, String name, Expression callArgs, boolean isMethod) {
             if (name==null) return;
             if (!optimizeMethodCall) return;
@@ -839,13 +859,13 @@ public class OptimizingStatementWriter extends StatementWriter {
                 target = selectConstructor(type, paraTypes);
                 if (target==null) return;
             }
-            
+
             StatementMeta meta = addMeta(expression);
             meta.target = target;
             meta.type = type;
             opt.chainShouldOptimize(true);
         }
-        
+
         private static MethodNode selectConstructor(ClassNode node, Parameter[] paraTypes) {
             List<ConstructorNode> cl = node.getDeclaredConstructors();
             MethodNode res = null;
@@ -870,7 +890,7 @@ public class OptimizingStatementWriter extends StatementWriter {
         public void visitClosureExpression(ClosureExpression expression) {
             return;
         }
-        
+
         @Override
         public void visitForLoop(ForStatement statement) {
             opt.push();
@@ -879,6 +899,6 @@ public class OptimizingStatementWriter extends StatementWriter {
             opt.pop(opt.shouldOptimize());
         }
     }
-    
+
 
 }
